@@ -156,8 +156,14 @@ public struct ErasedSequence<Element>: AsyncSequence {
 	}
 }
 
+#if swift(>=6.2)
+	public typealias AsyncSequenceWithSendableMetatype = AsyncSequence & SendableMetatype
+#else
+	public typealias AsyncSequenceWithSendableMetatype = AsyncSequence
+#endif
+
 extension AsyncThrowingStream where Element: Sendable, Failure == Error {
-	actor PermissiveActionLink<S: AsyncSequence> where S.Element == Element, S.AsyncIterator: Sendable {
+	actor PermissiveActionLink<S: AsyncSequenceWithSendableMetatype> where S.Element == Element, S.AsyncIterator: Sendable {
 		var iterator: S.AsyncIterator
 		let count: Int
 		var queued = [CheckedContinuation<Element?, Error>]()
@@ -205,6 +211,12 @@ protocol BackpressureProvider {
 	mutating func dequeue(_: Element)
 }
 
+#if swift(>=6.2)
+	typealias BackpressureProviderWithSendableMetatype = BackpressureProvider & SendableMetatype
+#else
+	typealias BackpressureProviderWithSendableMetatype = BackpressureProvider
+#endif
+
 final class CountedBackpressure<Element>: BackpressureProvider {
 	var count = 0
 	let max: Int
@@ -247,7 +259,7 @@ final class FileBackpressure: BackpressureProvider {
 	}
 }
 
-actor BackpressureStream<Element: Sendable, Backpressure: BackpressureProvider>: AsyncSequence where Backpressure.Element == Element {
+actor BackpressureStream<Element: Sendable, Backpressure: BackpressureProviderWithSendableMetatype>: AsyncSequence where Backpressure.Element == Element {
 	struct Iterator: AsyncIteratorProtocol {
 		let stream: BackpressureStream
 
@@ -344,6 +356,18 @@ actor BackpressureStream<Element: Sendable, Backpressure: BackpressureProvider>:
 		results = .error(error)
 		nextCondition?.signal()
 	}
+
+	@discardableResult
+	nonisolated func task<Success>(body: sending @escaping () async throws -> Success) -> Task<Success, Error> {
+		Task {
+			do {
+				return try await body()
+			} catch {
+				finish(throwing: error)
+				throw error
+			}
+		}
+	}
 }
 
 actor ConcurrentStream<Element: Sendable> {
@@ -606,7 +630,7 @@ public struct File: Sendable {
 	public let ino: Int
 	public let mode: Int
 	public let name: String
-	public var data = [ArraySlice<UInt8>]()
+	public internal(set) var data = [ArraySlice<UInt8>]()
 	var looksIncompressible = false
 
 	struct Identifier: Hashable {
@@ -807,13 +831,19 @@ public protocol StreamAperture {
 	associatedtype Next: StreamAperture
 	associatedtype Options: Sendable
 
-	static func transform(_: sending Input, options: Options?) -> Next.Input
+	static func transform(_: sending Input, options: Options?) -> sending Next.Input
 }
 
 protocol Decompressor {
 	static func decompress(data: [UInt8], decompressedSize: Int) throws -> [UInt8]
 }
 
+#if swift(>=6.2)
+	typealias DecompressorWithSendableMetatype = Decompressor & SendableMetatype
+#else
+	typealias DecompressorWithSendableMetatype = Decompressor
+#endif
+
 public enum DefaultDecompressor {
 	enum Zlib: Decompressor {
 		static func decompress(data: [UInt8], decompressedSize: Int) throws -> [UInt8] {
@@ -853,7 +883,7 @@ public enum DefaultDecompressor {
 	}
 }
 
-public enum XIP<S: AsyncSequence>: StreamAperture where S.Element: RandomAccessCollection, S.Element.Element == UInt8 {
+public enum XIP<S: AsyncSequenceWithSendableMetatype>: StreamAperture where S.Element: RandomAccessCollection, S.Element.Element == UInt8 {
 	public typealias Input = DataReader<S>
 	public typealias Next = Chunks
 
@@ -861,7 +891,7 @@ public enum XIP<S: AsyncSequence>: StreamAperture where S.Element: RandomAccessC
 		let zlibDecompressor: @Sendable ([UInt8], Int) throws -> [UInt8]
 		let lzmaDecompressor: @Sendable ([UInt8], Int) throws -> [UInt8]
 
-		init<Zlib: Decompressor, LZMA: Decompressor>(zlibDecompressor: Zlib.Type, lzmaDecompressor: LZMA.Type) {
+		init<Zlib: DecompressorWithSendableMetatype, LZMA: DecompressorWithSendableMetatype>(zlibDecompressor: Zlib.Type, lzmaDecompressor: LZMA.Type) {
 			self.zlibDecompressor = Zlib.decompress
 			self.lzmaDecompressor = LZMA.decompress
 		}
@@ -939,20 +969,16 @@ public enum XIP<S: AsyncSequence>: StreamAperture where S.Element: RandomAccessC
 		file.cap = file.position + contentSize
 	}
 
-	public static func transform(_ data: sending Input, options: Options?) -> Next.Input {
+	public static func transform(_ data: sending Input, options: Options?) -> sending Next.Input {
 		let options = options ?? Self.defaultOptions
 
 		let decompressionStream = ConcurrentStream<Void>(consumeResults: true)
 		let chunkStream = BackpressureStream(backpressure: CountedBackpressure(max: 16), of: Chunk.self)
 
-		Task {
+		chunkStream.task {
 			var content = data
 
-			do {
-				try await locateContent(in: &content, options: options)
-			} catch {
-				chunkStream.finish(throwing: error)
-			}
+			try await locateContent(in: &content, options: options)
 
 			let magic = "pbzx".utf8
 			try await UnxipError.throw(.invalid, if: await !content.read(magic.count).elementsEqual(magic))
@@ -1003,9 +1029,9 @@ public enum Chunks: StreamAperture {
 
 	public typealias Options = Never
 
-	public static func transform(_ chunks: sending Input, options: Options?) -> Next.Input {
+	public static func transform(_ chunks: sending Input, options: Options?) -> sending Next.Input {
 		let fileStream = BackpressureStream(backpressure: FileBackpressure(maxSize: 1_000_000_000), of: File.self)
-		Task {
+		fileStream.task {
 			var iterator = chunks.makeAsyncIterator()
 			var chunk = try await UnxipError.throw(.truncated, ifNil: await iterator.next())
 			var position = chunk.buffer.startIndex
@@ -1105,7 +1131,22 @@ public enum Files: StreamAperture {
 		.init(compress: true, dryRun: false)
 	}
 
-	public static func transform(_ files: sending Input, options: Options?) -> Next.Input {
+	private static func measureFilesystemOperation<T>(
+		on file: File,
+		named name: StaticString,
+		operation: () -> T
+	) -> T {
+		#if PROFILING
+			let id = OSSignpostID(log: filesystemLog)
+			os_signpost(.begin, log: filesystemLog, name: name, signpostID: id, "Starting %s for %s", name.description, file.name)
+			defer {
+				os_signpost(.end, log: filesystemLog, name: name, signpostID: id, "Completed")
+			}
+		#endif
+		return operation()
+	}
+
+	public static func transform(_ files: sending Input, options: Options?) -> sending Next.Input {
 		let options = options ?? Self.defaultOptions
 		let taskStream = ConcurrentStream<Void>()
 
@@ -1125,7 +1166,7 @@ public enum Files: StreamAperture {
 		}
 		let completion = Completion()
 
-		Task {
+		completion.completionStream.task {
 			let compressionStream = ConcurrentStream<[UInt8]?>(consumeResults: true)
 
 			var hardlinks = [File.Identifier: (String, Task<Void, Error>)]()
@@ -1134,18 +1175,6 @@ public enum Files: StreamAperture {
 			for try await file in files {
 				await completion.waitForCompletions()
 
-				@Sendable
-				func measureFilesystemOperation<T>(named name: StaticString, _ operation: () -> T) -> T {
-					#if PROFILING
-						let id = OSSignpostID(log: filesystemLog)
-						os_signpost(.begin, log: filesystemLog, name: name, signpostID: id, "Starting %s for %s", name.description, file.name)
-						defer {
-							os_signpost(.end, log: filesystemLog, name: name, signpostID: id, "Completed")
-						}
-					#endif
-					return operation()
-				}
-
 				@Sendable
 				func warn(_ result: CInt, _ operation: String) {
 					if result != 0 {
@@ -1166,7 +1195,7 @@ public enum Files: StreamAperture {
 				@Sendable
 				func setStickyBit(on file: File) {
 					if file.sticky {
-						measureFilesystemOperation(named: "chmod") {
+						measureFilesystemOperation(on: file, named: "chmod") {
 							warn(chmod(file.name, mode_t(file.mode)), "Setting sticky bit on")
 						}
 					}
@@ -1193,7 +1222,7 @@ public enum Files: StreamAperture {
 							return
 						}
 
-						measureFilesystemOperation(named: "link") {
+						measureFilesystemOperation(on: file, named: "link") {
 							warn(link(original, file.name), "linking")
 						}
 					}
@@ -1210,7 +1239,7 @@ public enum Files: StreamAperture {
 								return
 							}
 
-							measureFilesystemOperation(named: "symlink") {
+							measureFilesystemOperation(on: file, named: "symlink") {
 								warn(symlink(String(data: Data(file.data.map(Array.init).reduce([], +)), encoding: .utf8)!, file.name), "symlinking")
 							}
 							setStickyBit(on: file)
@@ -1224,7 +1253,7 @@ public enum Files: StreamAperture {
 								return
 							}
 
-							measureFilesystemOperation(named: "mkdir") {
+							measureFilesystemOperation(on: file, named: "mkdir") {
 								warn(mkdir(file.name, mode_t(file.mode & 0o777)), "creating directory at")
 							}
 							setStickyBit(on: file)
@@ -1249,7 +1278,7 @@ public enum Files: StreamAperture {
 									return
 								}
 
-								let fd = measureFilesystemOperation(named: "open") {
+								let fd = measureFilesystemOperation(on: file, named: "open") {
 									open(file.name, O_CREAT | O_WRONLY, mode_t(file.mode & 0o777))
 								}
 								if fd < 0 {
@@ -1257,7 +1286,7 @@ public enum Files: StreamAperture {
 									return
 								}
 								defer {
-									measureFilesystemOperation(named: "close") {
+									measureFilesystemOperation(on: file, named: "close") {
 										warn(close(fd), "closing")
 									}
 									setStickyBit(on: file)
@@ -1277,7 +1306,7 @@ public enum Files: StreamAperture {
 									// TODO: handle partial writes smarter
 									repeat {
 										written = data.withUnsafeBytes { data in
-											measureFilesystemOperation(named: "pwrite") {
+											measureFilesystemOperation(on: file, named: "pwrite") {
 												pwrite(fd, data.baseAddress, data.count, off_t(position))
 											}
 										}
@@ -1313,29 +1342,25 @@ public enum Disk: StreamAperture {
 
 	public typealias Options = Never
 
-	public static func transform(_ files: Input, options: Options?) -> Next.Input {
+	public static func transform(_ files: Input, options: Options?) -> sending Next.Input {
 		fatalError()
 	}
 }
 
 public struct UnxipStream<T: StreamAperture> {
-	public static func xip<S: AsyncSequence>(wrapping: S) -> UnxipStream<XIP<S>> where S.Element: RandomAccessCollection, S.Element.Element == UInt8 {
-		return .init()
-	}
-
-	public static func xip<S: AsyncSequence>(input: DataReader<S>) -> UnxipStream<XIP<S>> where S.Element: RandomAccessCollection, S.Element.Element == UInt8 {
-		return .init()
-	}
-
-	public static var chunks: UnxipStream<Chunks> {
+	public static func xip<S: AsyncSequence>() -> UnxipStream<XIP<S>> where S.Element: RandomAccessCollection, S.Element.Element == UInt8 {
 		.init()
 	}
 
-	public static var files: UnxipStream<Files> {
+	public static func chunks() -> UnxipStream<Chunks> {
 		.init()
 	}
 
-	public static var disk: UnxipStream<Disk> {
+	public static func files() -> UnxipStream<Files> {
+		.init()
+	}
+
+	public static func disk() -> UnxipStream<Disk> {
 		.init()
 	}
 }
@@ -1346,8 +1371,7 @@ public struct Unxip {
 	}
 
 	public static func makeStream<Start: StreamAperture, End: StreamAperture>(from start: UnxipStream<Start>, to end: UnxipStream<End>, input: sending Start.Input, _ option1: Start.Options? = nil, _ option2: Start.Next.Options? = nil) -> End.Input where Start.Next.Next == End {
-		let input = Start.transform(input, options: option1)
-		return Start.Next.transform(input, options: option2)
+		return Start.Next.transform(Start.transform(input, options: option1), options: option2)
 	}
 
 	public static func makeStream<Start: StreamAperture, End: StreamAperture>(from start: UnxipStream<Start>, to end: UnxipStream<End>, input: sending Start.Input, _ option1: Start.Options? = nil) -> End.Input where Start.Next == End {
@@ -1360,7 +1384,7 @@ public struct Unxip {
 	}
 }
 
-extension AsyncSequence where Element: Sendable, AsyncIterator: Sendable {
+extension AsyncSequence where Element: Sendable, AsyncIterator: Sendable, Self: AsyncSequenceWithSendableMetatype {
 	public func lockstepSplit() -> (AsyncThrowingStream<Element, Error>, AsyncThrowingStream<Element, Error>) {
 		let pal = AsyncThrowingStream.PermissiveActionLink<Self>(iterator: makeAsyncIterator(), count: 2)
 
@@ -1389,7 +1413,7 @@ extension AsyncSequence where Element: Sendable, AsyncIterator: Sendable {
 				("s", "statistics", "Print statistics on completion."),
 				("v", "verbose", "Print xip file contents."),
 			]
-			static let version = "3.1"
+			static let version = "3.2"
 
 			var input: String?
 			var output: String?
@@ -1467,7 +1491,7 @@ extension AsyncSequence where Element: Sendable, AsyncIterator: Sendable {
 		}
 
 		actor Statistics {
-			static let byteCountFormatter: ByteCountFormatter = {
+			let byteCountFormatter: ByteCountFormatter = {
 				let byteCountFormatter = ByteCountFormatter()
 				byteCountFormatter.allowsNonnumericFormatting = false
 				return byteCountFormatter
@@ -1540,9 +1564,9 @@ extension AsyncSequence where Element: Sendable, AsyncIterator: Sendable {
 			}
 
 			func printStatistics() {
-				print("Read \(Self.byteCountFormatter.string(fromByteCount: Int64(read)))", terminator: "")
+				print("Read \(self.byteCountFormatter.string(fromByteCount: Int64(read)))", terminator: "")
 				if let total = total {
-					print(" (out of \(Self.byteCountFormatter.string(fromByteCount: Int64(total))))", terminator: "")
+					print(" (out of \(self.byteCountFormatter.string(fromByteCount: Int64(total))))", terminator: "")
 				}
 				if #available(macOS 13.0, iOS 16.0, watchOS 9.0, tvOS 16.0, *),
 					let start = start as? ContinuousClock.Instant
@@ -1590,7 +1614,7 @@ extension AsyncSequence where Element: Sendable, AsyncIterator: Sendable {
 				}
 			}
 
-			for try await file in Unxip.makeStream(from: .xip(wrapping: input), to: .disk, input: DataReader(data: input), nil, nil, .init(compress: options.compress, dryRun: options.dryRun)) {
+			for try await file in Unxip.makeStream(from: .xip(), to: .disk(), input: DataReader(data: input), nil, nil, .init(compress: options.compress, dryRun: options.dryRun)) {
 				await statistics.note(file)
 				if options.verbose {
 					print(file.name)
